import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("split")
    def test_split(self):
        self.gprbuild()
        self.runexec("obj/split", "split.expected")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep
import os.path


class Test(GNATCOLL_TestCase):
    @chdir("win")
    def test_windows(self):
        self.gprbuild()
        self.runexec([os.path.join("obj","test_win"), "unix.txt"],
                     "test.out", msg="File in Unix format")
        file("win.txt", "wb").write(
            file("unix.txt").read().replace("\n", "\r\n"))
        self.runexec([os.path.join("obj","test_win"), "win.txt"],
                     "test.out", msg="File in Windows format")



#!/usr/bin/env python
"""
Runs the gnatcoll testsuite through the unittesting framework in python,
and add support for reporting events to GAIA.

You can specify the names of one or more testsuites to execute directly on
the command line, for instance
    xref  json.Test.test_json1
Do not forget the name of the test class

"""

import support
import os
import sys
import unittest

os.environ["PATH"] = \
    os.path.join(os.getcwd(), "..", "local_install", "lib") \
    + support.pathsep \
    + os.path.join(os.getcwd(), "..", "local_install", "bin") \
    + support.pathsep + os.getenv("PATH", "")

os.environ["GPR_PROJECT_PATH"] = \
    os.path.join(os.getcwd(), "..", "local_install", "lib", "gnat") + \
    support.pathsep + \
    os.path.join(os.getcwd(), "..", "src") + \
    support.pathsep + \
    os.getenv("GPR_PROJECT_PATH", "")

os.environ["LD_LIBRARY_PATH"] = \
    os.path.join(os.getcwd(), "..", "local_install", "lib") + \
    support.pathsep + \
    os.getenv("LD_LIBRARY_PATH", "")

testnames = set()
suites = unittest.TestSuite()
loader = unittest.TestLoader()

support.verbosity = 0
for v in sys.argv[1:]:
    if v == "-v":
        support.verbosity += 1
    else:
        testnames.add(v)

all_testsuites = set([
    'iconv', 'readline', 'config', 'traces', 'scripts', 'mmap_tests',
    'email', 'paragraph_filling', 'geometry', 'templates',
    'vfs', 'tribools', 'refcount', 'pools', 'storage_pools',
    'sql', 'projects', 'xref', 'json', 'utils'

    # Disable ravenscar tests, not reliable (H329-004)
    # Disable GMP tests, we get undefined symbols on saumur (IA23-022)
])

if testnames:
    n = [n.split(".")[0] for n in testnames]
    all_testsuites.intersection_update(n)

for s in sorted(all_testsuites):
    __import__(s)
    m = sys.modules[s]

    if not testnames or s in testnames:
        suite = loader.loadTestsFromModule(m)  # No filtering
    else:
        names = [n.split(".", 1)[1] for n in testnames
                 if n.startswith(s + ".")]
        suite = loader.loadTestsFromNames(names=names, module=m)
    suites.addTest(suite)

runner = support.GNATCOLL_TestRunner(
    verbosity=support.verbosity, stream=sys.stderr)
runner.separator2 = None
runner.run(suites)



import unittest
import support
import shutil
import os
import sys
import time

debug = False


def build_test(name):
    @support.chdir(name)
    def internal(self):
        # If the first line of the file indicates the project, load it
        try:
            f = file("test.cmd")
            lines = f.readlines()
        except:
            print "No %s/test.cmd" % name
            return

        self.unlink_if_exists(["gnatinspect.db",
                               "gnatinspect.db-wal",
                               "gnatinspect.db-shm"])

        dbswitch = "--db=:memory:"
        # dbswitch = "--db=gnatinspect.db"
        switches = ["gnatinspect", "--basenames", "-v", "--file=test.cmd",
                    "--lead=#"]

        if os.path.isfile("gnatdebug"):
            switches.append("--tracefile=gnatdebug")
        else:
            switches.append("--tracefile=../gnatdebug")

        if debug:
            switches = ["gdb", "-args"] + switches

        for line in lines:
            if line.startswith("--TESTNOCOMPILE "):
                extra = line.split()[1:]
                switches.extend(extra)
            elif line.startswith("--RM "):
                # remove a specific file if it exists
                self.unlink_if_exists(line.split()[1:])
            elif line.startswith("--CP "):
                # copy a file to a destination file
                files = line.split()[1:]
                shutil.copy(files[0], files[1])
            elif line.startswith("--TEST "):
                # Run gprbuild explicitly, and share its arguments with
                # gnatinspect
                extra = line.split()[1:]
                switches.extend(extra)
                self.runexec(["gprbuild", "-c", "-q"] + extra
                             + ["-cargs:C", "-fsyntax-only", "-fdump-xref"]
                             + ["-cargs:C++", "-fsyntax-only", "-fdump-xref"])
            elif line.startswith("--GNATINSPECT "):
                # Extra switches for gnatinspect
                switches.extend(line.split()[1:])
            elif line.startswith("--TESTSETUP "):
                self.runexec(line.split()[1:])
            elif line.startswith("--TESTDB "):
                dbswitch = "--db=%s" % (line.split()[1], )

        switches.append(dbswitch)

        # Do we have a project ?
        has_project = False
        dummy_created = False
        for s in switches:
            has_project = has_project or ".gpr" in s

        if not has_project:
            if os.path.isfile("default.gpr"):
                switches.append("default.gpr")
                self.runexec(["gprbuild", "-c", "-q", "default.gpr"]
                             + ["-cargs:C", "-fsyntax-only", "-fdump-xref"]
                             + ["-cargs:C++", "-fsyntax-only", "-fdump-xref"])
            else:
                dummy_created = True
                file("dummy.gpr", "w").write("""
project Dummy is
   for Source_Dirs use (".");
   for Object_Dir use ".";
end Dummy;
""")
                switches.append("dummy.gpr")

        self.runexec(switches, expected="test.cmd", capture_output=not debug)

        if os.path.exists('gnatinspect.db-wal'):
            sys.stdout.write('gnatinspect.db-wal was not removed')
        if dummy_created:
            os.unlink("dummy.gpr")

    internal.__name__ = "test_%s" % name
    return internal


class Test(support.GNATCOLL_TestCase):
    @support.chdir("parse_all")
    def test_parse_all(self):
        if os.path.isfile("../../../../gps/gps.gpr"):
            self.unlink_if_exists(["gnatinspect.db",
                                   "gnatinspect.db-wal",
                                   "gnatinspect.db-shm"])

            start = time.time()
            out = self.runexec(["gnatinspect", "-Pdefault.gpr", "--exit"])
            sys.stdout.write("parse gps.gpr: %.3fs\n" % (time.time() - start,))

            if os.path.exists('gnatinspect.db-wal'):
                sys.stdout.write('gnatinspect.db-wal was not removed')

            start = time.time()
            out = self.runexec(
                ["gnatinspect", "--force", "-Pdefault.gpr", "--exit"])
            sys.stdout.write(
                "update all gps.gpr: %.3fs\n" % (time.time() - start,))

            if os.path.exists('gnatinspect.db-wal'):
                sys.stdout.write('gnatinspect.db-wal was not removed')

        else:
            sys.stdout.write(
                "xref.test_parse_all not run (gps.gpr not found)\n")

    @support.chdir("M911_006")
    def test_M911_006(self):
        out = self.runexec(["sh", "test.cmd"], expected="test.out")

    # requires -lpthread on the command line on linux
    @support.requires_sqlite
    @support.chdir('MA27-009')
    def DISABLED_test_MA27_009(self):
        self.gprbuild('build.gpr')
        self.gcc('parent.ads')
        self.runexec('query', expected='test.out')

# Decorators for the tests

decorators = {

    # Windows uses a different s-memory.adb file, so the output is different
    # there.
    "test_L406-006-runtime": [support.requires_not_windows],

    # gcc and g++ output different xref information on some platforms LA02-019
    "test_cpp_class": [support.requires_not_windows, support.requires_cpp],
    "test_cpp_instance_variable": [support.requires_not_windows, support.requires_cpp],
    "test_cpp_ref1": [support.requires_not_windows],
    "test_cpp_ref2": [support.requires_not_windows],
    "test_cpp_ref3": [support.requires_not_windows],

    "test_cpp_abstract": [support.requires_cpp],
    "test_cpp_class": [support.requires_cpp],
    "test_cpp_enum": [support.requires_cpp],
    "test_cpp_forward_decl": [support.requires_cpp],
    "test_cpp_namespace1": [support.requires_cpp],
    "test_cpp_operators": [support.requires_cpp],
    "test_cpp_overloading": [support.requires_cpp],
    "test_cpp_separate_files1": [support.requires_cpp],
    "test_cpp_union": [support.requires_cpp],
    "test_doc1": [support.requires_cpp],
    "test_L821-017-import": [support.requires_cpp],
    "test_F630-024": [support.requires_cpp],
    "test_LB22-026": [support.requires_cpp],
    "test_LB27-003": [support.requires_cpp],
    "test_M826-001": [support.requires_cpp],
}

# Create tests dynamically (each subdirectory is assumed to contain a test)

for base in os.listdir(Test.__module__):
    if os.path.isdir(os.path.join(Test.__module__, base)):
        name = "test_%s" % base
        if base != ".svn" \
           and not hasattr(Test, name) \
           and base not in ("MA27-009"):
            func = build_test(base)
            if decorators.get(name, None) is not None:
                for d in decorators[name]:
                    func = d(func)

            setattr(Test, name, func)




import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("bools1")
    def test_bools1(self):
        self.gprbuild()
        self.runexec("obj/test_tribools", "test_tribools.expected")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep, requires_not_windows
import os


class Test(GNATCOLL_TestCase):
    @chdir("vfs1")
    def test_vfs1(self):
        self.gprbuild()
        self.runexec("obj/test_vfs", "test_vfs.expected")

    @requires_not_windows
    @chdir("links1")
    def test_links1(self):
        self.gprbuild()

        file("hello.orig", "w").write("This is the original")
        try:
            os.unlink("hello.adb")
        except:
            pass
        os.symlink("hello.orig", "hello.adb")

        self.runexec("test_links", "")

        orig = file("hello.orig").read()
        self.assertEqual(orig, "This is the new version")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep, requires_iconv
import os


class Test(GNATCOLL_TestCase):
    @requires_iconv
    @chdir("iconv1")
    def test_iconv1(self):
        self.gprbuild()
        self.runexec("obj/test_iconv", "")
        self.runexec("obj/test_bad_charset", "")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep
import os


class Test(GNATCOLL_TestCase):
    @chdir("pools1")
    def test_pools1(self):
        self.gprbuild()
        self.runexec("obj/testpools", "test.out")



import GPS
import sys

def test_myclass():
    sys.stdout.write("start test_myclass\n")
    sys.stdout.flush()

    c = GPS.MyClass()
    sys.stdout.write("c.refcount=%d\n" % (sys.getrefcount(c), ))
    sys.stdout.flush()
    c = None

    c2 = GPS.MyClass.get()
    sys.stdout.write("c2.refcount=%d\n" % (sys.getrefcount(c2), ))
    sys.stdout.flush()
    c2 = None

    sys.stdout.write("finish test_myclass\n")
    sys.stdout.flush()

def test_cache():
    sys.stdout.write("start test_cache\n")
    sys.stdout.flush()

    c = GPS.Cache.get()
    sys.stdout.write("c.value=%s  c.refcount=%d\n" % (c.value(), sys.getrefcount(c)))
    sys.stdout.flush()
    c.foo = 1
    c = None

    c2 = GPS.Cache.get()
    sys.stdout.write("c2.foo=%s\n" % getattr(c2, "foo", None))
    sys.stdout.write("c2.value=%s  c2.refcount=%d\n" % (c2.value(), sys.getrefcount(c2)))
    sys.stdout.flush()

    c2.destroy()    # Simulate a free from Ada
    c2 = None       # The last ref was hold by python

    sys.stdout.write("finish test_cache\n")
    sys.stdout.flush()

test_myclass()
sys.stdout.write("\n")
test_cache()



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("read1")
    def test_read1(self):
        # Only test that we can compile
        self.gprbuild()



#!/usr/bin/env python

import os
import os.path
import json

TEN_MILL = 10000000

if not os.path.isdir('json'):
    os.makedirs('json')

def save(fname, data):
    with file(fname,'w') as f:
        f.write(json.dumps(data))

# 10 meg
data = []
for x in xrange(3000):
    data.append(
        {
            "a": "b"*1024,
            "c":"d"*2048,
            "e":"f"*512,
        })
save("json/1.json", data)

# 29 meg
data = []
for x in xrange(3):
    data.append( "a"*TEN_MILL);
save("json/2.json", data)

# 57 meg
data = []
for x in xrange(3):
    data.append( {"a"*TEN_MILL:"b"*TEN_MILL} )
save("json/3.json", data)

# 19 meg
# very large float
with file("json/4.json",'w') as f:
    f.write ("[")
    f.write ("1"*TEN_MILL+"."+"3"*TEN_MILL)
    f.write ("]")

# 10 meg
# generate a very large int
with file("json/5.json",'w') as f:
    f.write ("[")
    f.write ("5"*TEN_MILL)
    f.write ("]")

# bad booleans
with file("json/6.json",'w') as f:
    f.write ("[")
    f.write ("tru"+"e"*TEN_MILL)
    f.write (",fals"+"e"*TEN_MILL)
    f.write ("]")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep, requires_windows
import os


class Test(GNATCOLL_TestCase):
    @chdir("prj1")
    def test_prj(self):
        self.gprbuild()
        self.runexec("obj/test_projects", "")

    @chdir("K302-015")
    def test_K302_015(self):
        self.gprbuild("iter.gpr")
        self.runexec("obj/iter", "test.out", sorted=True)

    @chdir("K712-015")
    def test_K712_015(self):
        self.gprbuild("testit.gpr")
        self.runexec("test_it", "test.out")

    @chdir("K926-026")
    def test_K926_026(self):
        self.gprbuild("prj.gpr")
        self.runexec("main", "")

    @chdir("KB30-016")
    def test_KB30_016(self):
        self.gprbuild("testit.gpr")
        self.runexec("test_it", "")

    @chdir("L801-020")
    def test_L801_020(self):
        self.gprbuild("default.gpr")
        self.runexec("obj/test_projects", "")

    @chdir("c_sources")
    def test_c_sources(self):
        self.gprbuild("default.gpr")
        self.runexec("obj/test_c", "test.out")

    @chdir("M320-003")
    def test_M320_003(self):
        # We want the same error messages in gprbuild and GNATCOLL
        # (GNATCOLL used to drop the missing withs)
        self.gprbuild("xxxx.gpr")
        self.runexec("main", "test.out")

    @chdir("objects")
    def test_objects(self):
        # Test that we can correctly find the location of ALI files
        # in the presence of extending projects
        self.runexec(["sh", "test.cmd"], "")

    @chdir("objects2")
    def test_objects2(self):
        # Test that we can correctly find the location of ALI files
        # when using xref_subdirs
        self.runexec(["sh", "test.cmd"], "")
		
    @chdir("aggregate")
    def test_aggregate(self):
        # Test that we can load aggregate project, get sorce files
        # and iterate through project hierarchy
        self.gprbuild("default.gpr")
        self.runexec("main", "")

    @chdir("aggregate_info")
    def test_aggregate_info(self):
        # Test that we can get File_Info via Info_Set and results
        # are correct
        self.gprbuild("default.gpr")
        self.runexec("main", "") 

    @requires_windows
    @chdir("aggregate_info_win")
    def test_aggregate_info_win(self):
        # Test on mixed casing passed to Info
        self.gprbuild("default.gpr")
        self.runexec("main", "")

    @chdir("aggregate_create")
    def test_aggregate_create(self):
        # Test on Create for aggregate projects
        self.gprbuild("aggr.gpr")
        self.gprbuild("default.gpr")
        self.runexec("main", "")

    @chdir("aggregate_ali")
    def test_aggregate_ali(self):
        # Test that ALI files with same base names but from different
        # aggregated projects do not exclude each other
        self.gprbuild("p2_2.gpr")
        self.gprbuild("p2.gpr")
        self.gprbuild("p1.gpr")
        self.gprbuild("default.gpr")
        self.runexec("main", "")




import unittest
from support import GNATCOLL_TestCase, chdir, pathsep, requires_not_windows
import os
import sys


def requires_email_data(func):
    def wrapped(*args):
        if os.path.isdir("email/email_data"):
            return func(*args)
        if hasattr(unittest, "SkipTest"):
            raise unittest.SkipTest("No email_data available")
    return wrapped


class Test(GNATCOLL_TestCase):
    def setUp(self):
        # See man tzset(3) for more information on the format of this variable
        # Let's go to newzealand.
        # This fails on Windows, so disable for now

        os.environ["TZ"] = "NZS-12"

    @requires_not_windows
    @chdir("email3")
    def test_email3(self):
        self.gprbuild()
        self.runexec("obj/test_email3", "test_email3.expected")

    @requires_email_data
    @chdir("email4")
    def test_multiple_files_in_single_mbox(self):
        self.gprbuild()
        self.runexec("obj/test_email4", "")

    @requires_email_data
    @chdir("email5")
    def test_threading(self):
        self.gprbuild()
        self.runexec("obj/test_email5", "test_email5.expected")

    # Disabled on windows, since the env. variable TZ is not taken into account
    # and thus the results are different
    @requires_not_windows
    @requires_email_data
    @chdir("email1")
    def test_api(self):
        self.gprbuild()
        self.runexec("obj/test_email", "test_email.expected")

    @requires_email_data
    @chdir("email6")
    def test_splitted_headers(self):
        self.gprbuild()
        self.runexec("obj/test_email6", "test_email6.expected")

    @chdir("encode")
    def test_encode(self):
        self.gprbuild()
        self.runexec("obj/encode_test", "")

    # Do not run on Windows, we do not know how to setup the proper
    # timezones, and the TZ environment variable is not enough
    # (see MA16-015)
    @requires_not_windows
    @chdir("date")
    def test_date(self):
        self.gprbuild()
        self.runexec("obj/test_email_date", "test_email_date.expected")

    @chdir("L619-017")
    def test_L619_017(self):
        self.gprbuild()
        self.runexec("obj/parse", "test.out")

    @chdir("M409-021")
    def test_M409_021(self):
        self.gprbuild()
        self.runexec("obj/process_msg", "test.out")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep
import sys


def requires_python(func):
    def wrapped(*args):
        if file("../Makefile.conf").read().find("WITH_PYTHON=yes") > 0:
            return func(*args)
        if hasattr(unittest, "SkipTest"):
            raise unittest.SkipTest("No python support")
    wrapped.__name__ = func.__name__
    return wrapped


class Test(GNATCOLL_TestCase):
    @requires_python
    @chdir("completion")
    def test_completion(self):
        self.gprbuild()
        self.runexec("obj/testapi", "testapi.out")

    @requires_python
    @chdir("python")
    def test_python1(self):
        self.gprbuild()
        self.runexec(["obj/testsuite", "python", "scripts1.py"],
                     "scripts1.out")

    @requires_python
    @chdir("python")
    def test_python3(self):
        self.gprbuild()
        self.runexec(["obj/testsuite", "python", "scripts3.py"],
                     "scripts3.out")

    @requires_python
    @chdir("python_leaks")
    def test_python_leaks(self):
        self.gprbuild()
        self.runexec(["obj/leaks"], "test.out")

    @chdir("python")
    def test_shell(self):
        self.gprbuild()
        self.runexec(["obj/testsuite", "shell", "scripts2.gsh"],
                     "scripts2.out")

    @requires_python
    @chdir("K318_018")
    def test_K318_018(self):
        self.gprbuild()
        self.runexec("obj/testpython", "K318_018.out")

    @requires_python
    @chdir("M701-028-dict-iter")
    def test_M701_028(self):
        self.gprbuild("pytestcase.gpr")
        self.runexec("bin/pytestcase", "test.out")



#!/usr/bin/env python
"""
Support package for the testsuite
"""

import unittest
import os
import shutil
import sys
import re
import subprocess
import difflib
import platform


exe_suffix = ''
pathsep = ":"

cygwin = 'uname' in os.__dict__ and os.uname()[0].find('CYGWIN') != -1
windows = cygwin or platform.system().find("Windows") != -1

if cygwin:
    exe_suffix = '.exe'
    pathsep = ":"
elif windows:
    exe_suffix = '.exe'
    pathsep = ";"

verbosity = 0

valgrind = None
redirect = True
if os.getenv("VALGRIND") == "leaks":
    valgrind = ["valgrind", "--tool=memcheck", "--num-callers=30",
                "-q",
                "--leak-check=full",
                "--show-reachable=no",
                "--leak-resolution=high",
                "--suppressions=%s/valgrind.supp" % os.getcwd(),
                "--suppressions=%s/valgrind-python.supp" % os.getcwd(),
                "--gen-suppressions=no"]
elif os.getenv("VALGRIND") == "yes":
    valgrind = ["valgrind", "--tool=memcheck", "--num-callers=30",
                "-q",
                "--suppressions=%s/valgrind.supp" % os.getcwd(),
                "--suppressions=%s/valgrind-python.supp" % os.getcwd(),
                "--gen-suppressions=no"]
elif os.getenv("GDB") == "yes":
    valgrind = ["gdb"]
    redirect = False


def find_on_path(execname):
    """Search for EXECNAME on the PATH, or return None"""

    dirs = os.getenv('PATH').split(pathsep)
    if pathsep != ":":
        # It seems to be difficult on Windows to know whether we are using
        # : or ;, so we just try both
        dirs.extend(os.getenv('PATH').split(":"))

    dirs.insert(0, os.getcwd())

    for dir in dirs:
        p = os.path.join(os.path.abspath(dir), execname)
        if os.path.exists(p):
            return p

        p = os.path.join(os.path.abspath(dir), execname + exe_suffix)
        if os.path.exists(p):
            return p

    return None


def cpp_supports_dump_xref():
    # Check whether we have "g++" next to "gcc", and assume it supports
    # "-fdump-xref" in this case
    p = find_on_path("gcc")
    if p:
        if os.path.exists(os.path.join(os.path.dirname(p), "g++")):
            return True
    return False


has_sqlite = file("../Makefile.conf").read().find("WITH_SQLITE=yes") > 0
has_postgres = file("../Makefile.conf").read().find("WITH_POSTGRES=yes") > 0
has_iconv = file(
    os.path.join(os.getcwd(), "../gnatcoll_shared.gpr")).read().find(
        'Iconv : Yes_No := "yes";') >= 0
has_cpp = cpp_supports_dump_xref()


def requires_not_windows(func):
    """Only execute the test if not running on windows"""
    def wrapped(*args):
        if not windows:
            return func(*args)
        if hasattr(unittest, "SkipTest"):
            raise unittest.SkipTest("Not supported on Windows")
    wrapped.__name__ = func.__name__
    return wrapped

	
def requires_windows(func):
    """Only execute the test if running on windows"""
    def wrapped(*args):
        if windows:
            return func(*args)
        if hasattr(unittest, "SkipTest"):
            raise unittest.SkipTest("Supported only on Windows")
    wrapped.__name__ = func.__name__
    return wrapped


def requires_cpp(func):
    """Only execute the test if c++ is available"""
    def wrapped(*args):
        if has_cpp:
            return func(*args)
        if hasattr(unittest, "SkipTest"):
            raise unittest.SkipTest("Requires a c++ compiler")
    wrapped.__name__ = func.__name__
    return wrapped


def requires_iconv(func):
    """Only execute the test if iconv is available"""
    def wrapped(*args):
        if has_iconv:
            return func(*args)
        if hasattr(unittest, "SkipTest"):
            raise unittest.SkipTest("Requires iconv")
    wrapped.__name__ = func.__name__
    return wrapped


def requires_sqlite(func):
    """A decorator that skips a test if sqlite is not installed"""
    def wrapped(*args):
        if has_sqlite:
            return func(*args)
        if hasattr(unittest, "SkipTest"):
            raise unittest.SkipTest("No sqlite support")
    wrapped.__name__ = func.__name__
    return wrapped


def requires_postgresql(func):
    """A decorator that skips a test if postgres is not installed"""
    def wrapped(*args):
        if has_postgres:
            return func(*args)
        if hasattr(unittest, "SkipTest"):
            raise unittest.SkipTest("No postgreSQL support")
    wrapped.__name__ = func.__name__
    return wrapped


class GNATCOLL_TestResult(unittest._TextTestResult):
    """Custom class to save test results for GAIA"""

    def __init__(self, stream, verbosity):
        super(GNATCOLL_TestResult, self).__init__(
            stream, 1, verbosity=verbosity)

        self.result_dir = os.path.join(os.getcwd(), "results")
        try:
            shutil.rmtree(self.result_dir, ignore_errors=True)
            os.makedirs(self.result_dir)
        except:
            pass

    def _do_out(self, test, short, err=None):
        name = test.id()
        file(os.path.join(self.result_dir, name + ".result"), "w") \
              .write(short)
        file(os.path.join(self.result_dir, "results"), "a") \
              .write(name + ":" + short + "\n")
        if err:
            file(os.path.join(self.result_dir, name + ".out"), "w") \
                  .write(str(err[1]))

    def addSuccess(self, test):
        super(GNATCOLL_TestResult, self).addSuccess(test)
        self._do_out(test, "OK:test passed")

    def addError(self, test, err):
        super(GNATCOLL_TestResult, self).addError(test, err)
        self._do_out(test, "FAILED:driver error", err)

    def addFailure(self, test, err):
        super(GNATCOLL_TestResult, self).addFailure(test, err)
        self._do_out(test, "DIFF:test failed", err)

    def addSkip(self, test, reason):
        if sys.version_info >= (2, 7):
            super(GNATCOLL_TestResult, self).addSkip(test, reason)
        self._do_out(test, "DEAD:" + reason)

    def addExpectedFailure(self, test, err):
        if sys.version_info >= (2, 7):
            super(GNATCOLL_TestResult, self).addExpectedFailure(test, err)
        self._do_out(test, "XFAIL:test should have failed", err=err)


class GNATCOLL_TestRunner(unittest.TextTestRunner):
    def _makeResult(self):
        return GNATCOLL_TestResult(self.stream, verbosity=self.verbosity)

    def run(self, test):
        result = self._makeResult()
        test(result)
        if verbosity >= 1:
            self.stream.write("(%d tests) " % result.testsRun)
        if not result.wasSuccessful():
            self.stream.writeln("FAILED")
        result.printErrors()


class GNATCOLL_TestCase(unittest.TestCase):

    def find_on_path(self, execname):
        """Search for EXECNAME on the PATH"""

        p = find_on_path(execname)
        if p is None:
            self.assertFalse(True, "Exec not found on PATH: %s" % execname)
            return execname
        return p

    def gprbuild(self, project="default.gpr"):
        self.runexec(['gprbuild', '-q', '-m', '-p', '-g', '-P%s' % project])

    def gcc(self, file):
        self.runexec(['gcc', '-c', '-gnatc', file])

    def runexec(self, cmdline, expected=None, msg="", sorted=False,
                capture_output=True,
                shell=False, expectedStatus=0, input=''):
        """Run the givne command, and compare its output to EXPECTED. The
           latter can also be the name of a file.
           First element in CMDLINE is converted to full path if necessary.
           No comparison done if EXPECTED is None.
           If SORTED is True, the actual output lines are first sorted before
           comparing with the expected output.
        """

        if isinstance(cmdline, str):
            cmdline = [cmdline]
        if os.path.exists(cmdline[0]):
            cmdline[0] = "./%s" % cmdline[0]
        else:
            cmdline[0] = self.find_on_path(cmdline[0])

        if valgrind and "gprbuild" not in cmdline[0]:
            cmdline = valgrind + cmdline

        if input:
            stdin = subprocess.PIPE
        else:
            stdin = None

        if capture_output and redirect:
            stdout = subprocess.PIPE
            stderr = subprocess.STDOUT
        else:
            stdout = stderr = None

        try:
            if verbosity >= 3:
                print "Running %s" % cmdline
            proc = subprocess.Popen(cmdline, stdout=stdout,
                                    shell=shell,
                                    stdin=stdin,
                                    stderr=stderr)
        except:
            self.assertTrue(False,
                "Could not execute %s" % cmdline)

        out = proc.communicate(input=input)[0]

        status = proc.wait()
        self.assertEqual(
            expectedStatus, status, "Failed to run %s\n%s\nstatus=%s" % (
                cmdline, out, status))

        if expected is not None:
            self.diff(expected=expected, actual=out, msg=msg, sorted=sorted)

        if verbosity >= 3:
            print out

        return out

    def diff(self, expected, actual, msg="", sorted=False):
        if os.path.isfile(expected):
            expected = file(expected).read()

        pwd = os.getcwd() + "/"
        actual = actual.replace(pwd, "<pwd>")

        if windows:
            pwd = pwd.replace("/", "\\")
            pwd = re.sub(r'^[cC]:', '', pwd)
            actual = actual.replace(pwd, "<pwd>")
            actual = re.sub(r'[cC]:<pwd>', '<pwd>', actual)
            actual = actual.replace("\r", "")

        act = actual.splitlines()

        if sorted:
            act.sort()

        d = difflib.unified_diff(expected.splitlines(), act)
        d = "\n".join(d)
        if d != "":
            self.assertTrue(False, msg + "\n" + d)

    def unlink_if_exists(self, files):
        if isinstance(files, str):
            files = [files]
        for f in files:
            try:
                os.unlink(f)
            except OSError:
                pass


def chdir(in_dir):
    """A decorator that will temporarily change to another directory
       The directory is relative to the containing module's directory.
    """
    def wrap(func):
        def wrapped(*args):
            try:
                pwd = os.getcwd()
                d = os.path.join(func.__module__, in_dir)
                if verbosity >= 3:
                    print "chdir %s" % d
                os.chdir(d)
                func(*args)
            finally:
                if verbosity >= 3:
                    print "chdir %s" % pwd
                os.chdir(pwd)
        wrapped.__name__ = func.__name__
        return wrapped
    return wrap



import difflib
import functools
import os.path
import shutil
import unittest

from support import GNATCOLL_TestCase
from mmap_tests.utils import *


TESTSUITE_ROOT = os.getcwd()
MMAP_ROOT = os.path.join(TESTSUITE_ROOT, 'mmap_tests')


def test_method(subdir):
    """
    Return a decorator for testcases' methods, used to make sure setUpClass is
    called once.

    This is needed as long as Python2.6 is supported (setUpClass is new in
    Python2.7)
    """
    def decorator(func):

        @functools.wraps(func)
        def wrapper(self):
            with Chdir(MMAP_ROOT):

                if not getattr(self, 'initialized', False):
                    self.setUpClass()
                self.initialized = True

                with Chdir(subdir):
                    return func(self)

        return wrapper

    return decorator


class MmapTestCase(GNATCOLL_TestCase):
    """Base class for GNATCOLL.Mmap testcases."""

    # Subdirectories to compile before starting tests.
    subdirs = []

    @classmethod
    def setUpClass(cls):
        """Build all test drivers in all subdirectories."""
        with Chdir(MMAP_ROOT):
            for subdir in cls.subdirs:
                with Chdir(subdir):
                    # TODO: adapt the support module to be able to call
                    # gprbuild() without an instance of GNATCOLL_TestCase.
                    gprbuild()

    def assertMultiLineEqual(self, expected, got):
        """
        Assert that expected equals got. Otherwise, print a diff on them.

        This is needed as long as Python2.6 is supported (assertMultiLineEqual
        is new in Python2.7)
        """
        self.assertEqual(expected, got,
            msg='\n'.join(difflib.unified_diff(expected, got,
                'expected', 'got')))


class LegacyTestCase(MmapTestCase):

    subdirs = ['map1', 'write']

    @test_method("map1")
    def test_full_mmap(self):
        self.runexec(["obj/test_mmap", "test_mmap.adb", "full", "mmap"])
        self.diff(expected="test_mmap.adb", actual=file("mmap.out").read())

    @test_method("map1")
    def test_partial_mmap(self):
        self.runexec(["obj/test_mmap", "test_mmap.adb", "partial", "mmap"])
        self.diff(expected="test_mmap.adb", actual=file("mmap.out").read())

    @test_method("map1")
    def test_full_read(self):
        self.runexec(["obj/test_mmap", "test_mmap.adb", "full", "read"])
        self.diff(expected="test_mmap.adb", actual=file("mmap.out").read())

    @test_method("map1")
    def test_partial_read(self):
        self.runexec(["obj/test_mmap", "test_mmap.adb", "partial", "read"])
        self.diff(expected="test_mmap.adb", actual=file("mmap.out").read())

    @test_method("map1")
    def test_length_32767(self):
        self.runexec(["obj/test_mmap", "mmap2.in", "full", "mmap"])
        self.diff(expected="mmap2.in", actual=file("mmap.out").read())

    @test_method("map1")
    def test_empty_file(self):
        self.runexec(["obj/test_mmap", "empty.txt", "full", "mmap"])
        self.diff(expected="", actual=file("mmap.out").read())

        self.runexec(["obj/test_mmap", "empty.txt", "full", "read"])
        self.diff(expected="", actual=file("mmap.out").read())


    @test_method("write")
    def test_write(self):
        self.gprbuild()
        file("mmap.in", "w").write(file("mmap1.in").read())
        self.runexec(["obj/test_mmap_write", "mmap"], "")
        self.diff(expected="mmap1.out", actual=file("mmap.in").read())

        file("mmap.in", "w").write(file("mmap1.in").read())
        self.runexec(["obj/test_mmap_write", "read"], "")
        self.diff(expected="mmap1.out", actual=file("mmap.in").read())


class MultipleRegionsTestCase(MmapTestCase):
    """
    Test the feature that allows one to map multiple regions of the same file
    simultaneously.
    """

    subdirs = ['multiple_regions']

    @classmethod
    def setUpClass(cls):
        with Chdir(MMAP_ROOT):
            super(MultipleRegionsTestCase, cls).setUpClass()
            with Chdir('multiple_regions'):
                cls.setup_files()
                cls.discover_mmap()

    @classmethod
    def setup_files(cls):
        # Prepare test material
        with open('empty.txt', 'wb'):
            pass
        with open('chars.txt', 'wb') as f:
            f.write('A')
            f.write('B' * 998)
            f.write('C')
            f.write('X')
            f.write('Y' * 998)
            f.write('Z')

    @classmethod
    def discover_mmap(cls):
        p = subprocess.Popen(
            ['obj/is_mmap_available'],
            stdout=subprocess.PIPE)
        stdout, stderr = p.communicate()
        cls.is_mmap_available = stdout.strip() == 'TRUE'

    @test_method('multiple_regions')
    def test_read_not_existing(self):
        """
        Trying to open a non-existing file should raise an
        Ada.IO_Exceptions.Name_Error.
        """
        self.run_driver('no-such-file', 'read', [],
            expected='Open: Name_Error')

    @test_method('multiple_regions')
    def test_write_not_existing(self):
        """
        Trying to open a non-existing file should raise an
        Ada.IO_Exceptions.Name_Error.
        """
        self.run_driver('no-such-file', 'write', [],
            expected='Open: Name_Error')

    @test_method('multiple_regions')
    def test_read_empty_file(self):
        """
        The mapped regions of an empty file must be at the required offset, but
        always empty.
        """
        self.run_driver('empty.txt', 'read', [
            Map(0,   0),
            Map(100, 0),
            Map(0,   100),
            Map(100, 100),
        ])

    @test_method('multiple_regions')
    def test_write_empty_file(self):
        self.run_driver('empty.txt', 'write', [
            Map(0, 0), Fill(1, 0, 'A'),
        ])

    @test_method('multiple_regions')
    def test_read_straigthforward(self):
        """
        Mapping regions fully contained in the file must work as expected.
        """
        self.run_driver('chars.txt', 'read', [
            Map(0,    1),
            Map(0,    1000),
            Map(1000, 2000),
        ])

    @test_method('multiple_regions')
    def test_read_remap_inside(self):
        """
        Remapping regions inside already existing regions must work as
        expected.
        """
        self.run_driver('chars.txt', 'read', [
            Map(0, 1000), Remap(0,    50),
            Map(0, 1000), Remap(100,  50),
            Map(0, 1000), Remap(900,  100),
            Map(0, 1000), Remap(800,  100),
        ])

    @test_method('multiple_regions')
    def test_read_remap_inside_rollback(self):
        """
        Remapping regions outside already existing regions must work as
        expected.
        """
        self.run_driver('chars.txt', 'read', [
            Map(0, 1000), Remap(0, 50), Remap(0, 1000),
        ])

    @test_method('multiple_regions')
    def test_read_remap_outside(self):
        self.run_driver('chars.txt', 'read', [
            Map(0, 1000), Remap(1000, 1000),
            Map(2000, 0), Remap(1000, 1000),
            Map(0, 1000), Remap(2000, 1000),
        ])

    @test_method('multiple_regions')
    def test_write_straightforward(self):
        self.run_driver('chars.txt', 'write', [
            Map(0, 1000),
            Fill(1,    0, 'I'),
            Fill(1,    1, 'J'),
            Fill(10,   1, 'K'),
            Fill(1000, 1, 'L'),
        ])

    @test_method('multiple_regions')
    def test_write_complete_fill(self):
        self.run_driver('chars.txt', 'write', [
            Map(0, 1000),
            Fill(1,    1000, 'I'),
            Fill(1,    1,    'J'),
            Fill(1000, 1,    'K'),
        ])

    @test_method('multiple_regions')
    def test_read_mutable(self):
        """
        Writing to mutable mapped regions of reading files should modify the
        buffer in memory, but not the underlying file.
        """
        self.run_driver('chars.txt', 'read', [
            Map(0, 1000, mutable=True),
            Fill(1, 1, 'A'),
        ], copy_file=True)
        self.assertMultiLineEqual(
            open('chars.txt', 'r').read(),
            open('temp.txt', 'r').read()
        )

    @test_method('multiple_regions')
    def test_read_remap_mutable(self):
        """Remapping some region must update its the mutability status."""
        self.run_driver('chars.txt', 'read', [
            Map(0, 1000, mutable=False),
            Remap(0, 1000, mutable=True),
            Fill(1, 1, 'A'),
        ], copy_file=True)
        self.assertMultiLineEqual(
            open('chars.txt', 'r').read(),
            open('temp.txt', 'r').read()
        )

    def run_driver(
        self, filename,
        mode, actions,
        expected=None,
        copy_file=False
    ):
        """Run the test driver.

        Make it map REGIONS from FILENAME. If EXPECTED is None, guess what is
        expected to output reading the FILENAME, otherwise use EXPECTED as the
        expected output. "Guessing" is basically just checking that regions
        memory mapping have the same behavior as regular file reading using
        Python's file manipulation primitives.

        If COPY_FILE is True, force the copy of FILENAME so that the test can
        harmlessly modify it.

        Test both mmap and manual read behaviors, and try both to read the file
        content before and after closing the mapped file.
        """

        # If the driver is supposed to write to the file, provide it with a
        # copy.
        if copy_file or (mode == 'write' and os.path.isfile(filename)):
            temp_filename = 'temp.txt'
        else:
            temp_filename = filename

        # GNATCOLL.Mmap does not allow working with mapped regions related to
        # writing closed files.
        close_after_read_set = (
            ('no-close-after-read', )
            if mode == 'write' else
            ('close-after-read', 'no-close-after-read')
        )

        for method in ('mmap', 'buffer'):
            for close_after_read in close_after_read_set:

                if temp_filename != filename:
                    shutil.copy(filename, temp_filename)

                # Format the expected driver output/file content if none is
                # provided. This can be done just reading the given file at the
                # given regions.
                if expected:
                    expected_output = expected
                    expected_content = None
                else:
                    regions, expected_content = interpret_actions(
                        temp_filename, mode, method, self.is_mmap_available,
                        actions)
                    expected_output = format_regions(temp_filename, regions)

                # Format the arguments to pass to the test driver.
                args = [
                    'obj/driver',
                    temp_filename, mode, method, close_after_read]
                args.extend(actions_to_args(actions))

                # Run it.
                self.runexec(args, expected=expected_output)

                # Then check that the content of the file is as expected.
                if temp_filename and expected_content is not None:
                    self.assertMultiLineEqual(
                        expected_content, open(temp_filename, 'r').read()
                    )



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep
import sys


class Test(GNATCOLL_TestCase):
    @chdir("traces1")
    def test_traces1(self):
        self.gprbuild()
        self.runexec("obj/traces1", "traces1.out")

        # Check that we do not link with python
        if sys.platform.find("linux") != -1:
            out = self.runexec(["ldd", "obj/traces1"])
            self.assertEqual(-1, out.find("libpython"),
                             "Should not link with python")

            out = self.runexec(["nm", "obj/traces1"])
            self.assertEqual(-1, out.find("Py"),
                             "Should not have python symbols")


    @chdir("traces2")
    def test_traces2(self):
        self.unlink_if_exists("outfile")
        self.gprbuild()
        self.runexec("obj/traces2", "traces2.out")
        self.diff(expected="outfile.out", actual=file("outfile").read())

    @chdir("traces4")
    def test_append_to_file(self):
        self.unlink_if_exists("traces4.out")
        self.unlink_if_exists("traces4.out2")
        self.gprbuild()
        self.runexec("obj/traces4", "")

        # Run a second time to test when we append to an existing file
        self.runexec("obj/traces4", "")
        self.diff(expected="traces4.expected",
                  actual=file("traces4.out").read())

    @chdir("wildcard")
    def test_wildcard(self):
        self.gprbuild()
        self.runexec("obj/wildcard", "")
        self.diff(expected="wildcard.expected",
                  actual=file("wildcard.out").read())
        self.diff(expected="wildcard.expected2",
                  actual=file("wildcard.out2").read())

    @chdir("L320-007")
    def test_L320_007(self):
        self.gprbuild()
        self.runexec("obj/main", "SUCCESS")

    @chdir("L401-004")
    def test_L401_004(self):
        self.gprbuild()
        self.runexec("obj/main", "test.out")

    @chdir("MA08-038")
    def test_MA08_038(self):
        self.gprbuild()
        self.runexec("nofile", "")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("fill1")
    def setUp(self):
        self.gprbuild()

    @chdir("fill1")
    def test_greedy(self):
        self.runexec(["obj/fill_ada_comments", "--greedy", "sem_ch3.adb"],
                      "sem_ch3.adb.greedy.known-good")

    @chdir("fill1")
    def test_knuth(self):
        self.runexec(["obj/fill_ada_comments", "--knuth", "sem_ch3.adb"],
                      "sem_ch3.adb.knuth.known-good")

    @chdir("fill1")
    def test_knuth_with_prefix(self):
        self.runexec(["obj/fill_ada_comments", "--knuth", "sem_ch3.adb",
                      "--prefix=##"],
                      "sem_ch3.adb.knuth_prefix.known-good")

    @chdir("fill1")
    def test_pretty(self):
        self.runexec(["obj/fill_ada_comments", "--pretty", "sem_ch3.adb"],
                      "sem_ch3.adb.pretty.known-good")



import M

print(M.hello())
print(M.hello("Manu"))

print("%1.2f" % M.print_float(1.2))

c = M.C1 (2)
print(c.method ("foo"))

print(c.id)
print(c.name)

c.prop = 5    # Calling setter
print(c.prop) # Calling getter

try:
    print(c.wo_prop) # Error
except AttributeError:
    print("Expected exception when getting wo_prop")



#!/usr/bin/env python

import os, os.path, re

pkg_re = re.compile ("^(private)?\s*package\s*(\S+)")

def recursive_ls (dir):
   """Return the list of ads files in dir and its subdirs"""
   result = set ()
   for f in os.listdir (dir):
      if f.endswith (".ads") \
         and f.startswith ("gnatcoll-"):

         private = False
         pkg = ""
         for l in file (os.path.join (dir, f)).readlines ():
            m = pkg_re.search (l)
            if m:
               private = m.group (1)
               pkg = m.group (2)
               break

         if not private:
            result.add ((pkg, os.path.splitext (f)[0]))

      elif os.path.isdir (os.path.join (dir, f)):
         result = result.union (recursive_ls (os.path.join (dir, f)))

   return result

list = recursive_ls ("../src")
out = file ("gnatcoll/runtime.py", "w")
out.write ("""XML = r'''<?xml version="1.0"?>
<GPS>
""")

for pkg, f in sorted (list):
   menu = pkg.replace (".", "/").replace ("_", "__")

   # Do we have a submenu ?
   in_front = False
   for pkg2, b in list:
       if b.startswith (f + "-"):
          item = menu [menu.rfind ("/") + 1:]
          menu = menu + "/&lt;" + item + "&gt;"
          break

   out.write ("""  <documentation_file>
     <shell>Editor.edit "%(file)s.ads"</shell>
     <descr>%(package)s</descr>
     <menu>/Help/GNAT Runtime/%(menu)s</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

""" % {"file":f, "menu":menu, "package":pkg})

out.write ("""</GPS>'''
import GPS
GPS.parse_xml(XML)
""")
out.close ()



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep
import os


class Test(GNATCOLL_TestCase):
    @chdir("ref1")
    def test_prj(self):
        self.gprbuild()
        self.runexec("obj/test_refcount", "test_refcount.out")

    #@chdir("ref2")
    #def test_ref2(self):
        #    self.gprbuild()
        #self.runexec("obj/test_refcount", "")



"""
This file makes the GNATCOLL documentation available from GPS
"""



import GPS
from . import runtime

XML = r"""<?xml version="1.0" ?>
<GPS>
  <doc_path>share/doc/gnatcoll</doc_path>
  <documentation_file>
     <name>html/index.html</name>
     <descr>Gnat Reusable Components User's Guide</descr>
     <category>GNAT</category>
     <menu before='About'>/Help/Gnat Components/Gnat Components User's Guide</menu>
  </documentation_file>
</GPS>
"""

GPS.parse_xml(XML)



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep, requires_sqlite, requires_postgresql
import os
import sys


class Test(GNATCOLL_TestCase):
    @chdir("sql1")
    def test_sql(self):
        self.runexec(['gnatcoll_db2ada', '-dbmodel', '../descr.txt'])
        self.gprbuild()
        self.runexec("obj/test_sql", "")

    @chdir("sql3")
    def test_sql3(self):
        self.unlink_if_exists(["test.db", "pkg.ads"])
        try:
            self.runexec(["sqlite3", "test.db"], input=file("test.sql").read())
            self.runexec(["gnatcoll_db2ada", "-dbname=test.db",
                          "-dbtype=sqlite", "-api=PKG"])
            self.gprbuild()
        except AssertionError:
            # sqlite3 not found
            pass

    @requires_sqlite
    @chdir("sql2")
    def test_sql2(self):
        try:
            os.unlink("tmp.db")
        except:
            pass

        # Create a tmp sqlite database
        self.runexec(['gnatcoll_db2ada',
                      '-dbmodel', '../descr.txt',
                      '-dbname', 'tmp.db',
                      '-dbtype', 'sqlite',
                      '-createdb'])
        self.runexec(['sqlite3', 'tmp.db'],
                     input="INSERT INTO config VALUES (0,'f');")
        # Extra info from it, to check that "max(id)" is correctly
        # interpreted. Also check that we know how to read the
        # schema from a live sqlite database.
        self.runexec(['gnatcoll_db2ada',
                      '-dbmodel', '../descr.txt',
                      '-dbname', 'tmp.db',
                      '-dbtype', 'sqlite',
                      '-api', 'DB',
                      '-enum', 'config,id,id,config_enum,',
                      '-var', 'max_config,config,max(id),,'])

        self.diff('db.ads.out', file('db.ads').read())

    @requires_sqlite
    @chdir("prepared")
    def test_prepared(self):
        self.unlink_if_exists("test.db")
        self.runexec(['gnatcoll_db2ada', '-dbmodel', 'dbschema.txt',
                      '-api', 'dbschema'])
        self.gprbuild()
        self.runexec("obj/prepared", "test.out")

    @requires_sqlite
    @chdir("../../examples/library")
    def test_check_doc(self):
        """Run the library example from the doc to detect errors"""
        self.runexec(["sh", "./build.sh"])

    @requires_postgresql
    @chdir("prepared_postgres")
    def test_postgres_prepared(self):
        self.runexec(['gnatcoll_db2ada',
                      '-dbmodel', '../prepared/dbschema.txt',
                      '-api', 'dbschema'])
        self.gprbuild()
        self.runexec("obj/preparedpsql", "test.out")

    @chdir("orm")
    def test_orm(self):
        os.environ["PATH"] = "." + pathsep + os.getenv("PATH")
        self.runexec(["gnatcoll_db2ada", "-orm=Queries",
                      "-api=ORM_Database", "-dbmodel=descr.txt"])
        self.runexec(["gnatcoll_db2ada", "-dot",
                      "-dbmodel=descr.txt",
                      "Group1:red:elements,properties"],
                    "test.out")
        self.gprbuild()

    @chdir("orm_views")
    def test_orm_views(self):
        os.environ["PATH"] = "." + pathsep + os.getenv("PATH")
        self.runexec(["gnatcoll_db2ada", "-orm=Queries",
                      "-api=ORM_Database", "-dbmodel=descr.txt"])
        self.gprbuild()

    @chdir("json")
    def test_json(self):
        self.runexec(["gnatcoll_db2ada", "-orm=Queries",
                      "-api=DB", "-dbmodel=descr.txt"])
        self.gprbuild()

    # requires -lpthread on command line
    @requires_sqlite
    @chdir("check_sqlite")
    def DISABLED_test_check_sqlite(self):
        self.gprbuild()
        self.runexec("sqlite_test", "")




from collections import namedtuple
import io
import itertools
import os
import os.path
import subprocess


def gprbuild(project='default.gpr'):
    """Compile the given project using GPRbuild."""
    subprocess.check_call(['gprbuild',
        '-q', '-m', '-p', '-g', '-we',
        '-P{0}'.format(project)])


class Chdir(object):
    """
    Context manager used to temporarily change the current directory.
    """

    def __init__(self, dirname):
        """
        Create a Chdir context manager.

        When entered, this context manager will change the current directory to
        DIRNAME. The previous current directory is restored when leaving this
        context manager.
        """
        self.requested_dir = dirname

    def __enter__(self):
        self.old_dir = os.getcwd()
        os.chdir(self.requested_dir)

    def __exit__(self, exc_type, exc_value, traceback):
        os.chdir(self.old_dir)


#
# Actions to be done by the MultipleRegions' test driver.
#

# Command use to create a mapped region. "offset" is relative to the beginning
# of the file, starting at 0.
_Map = namedtuple('Map', 'offset length mutable')
def Map(offset, length, mutable=False):
    return _Map(offset, length, mutable)

# Likewise, but remap the last created mapped region.
_Remap = namedtuple('Remap', 'offset length mutable')
def Remap(offset, length, mutable=False):
    return _Remap(offset, length, mutable)

# Fill the last mapped region with "filler" repeated as need. "offset" is
# relative to the begging of the mapped region, starting at 1. This pecularity
# is used to reflect the underlying "Str_Access" bounds.
# Note that this can be used with "mutable" regions on reading files.
Fill = namedtuple('Fill', 'offset length filler')


# Expected attributes of a mapped region.
Region = namedtuple('Region', 'offset last size mutable content')


def interpret_actions(filename, mode, method, is_mmap_available, actions):
    """
    Interpet ACTIONS on the given FILENAME, return the resulting regions and
    the content of the resulting file.
    """

    result = []
    f = io.BytesIO(open(filename, 'r').read())

    write = mode == 'write'
    mmap = method == 'mmap'

    def map_region(offset, length, mutable):
        f.seek(offset)
        content = f.read(length)
        size = len(content)
        last = max(0, offset + size)
        mutable = mutable or write or not mmap or not is_mmap_available
        return Region(offset, last, size, mutable, content)

    for action in actions:
        if isinstance(action, _Map):
            result.append(map_region(
                action.offset, action.length,
                action.mutable))

        elif isinstance(action, _Remap):
            result[-1] = map_region(
                action.offset, action.length,
                action.mutable)

        elif isinstance(action, Fill):
            region = result[-1]

            # Writing overflow is not catched by GNATCOLL.Mmap: this is a real
            # error.
            if action.offset + action.length - 1 > region.size:
                raise ValueError('{0} goes out of {1}'.format(action, region))

            f.seek(region.offset + action.offset - 1)
            bounded_filler = ''.join(itertools.islice(
                    itertools.cycle(action.filler),
                    0, action.length
            ))
            f.write(bounded_filler)

            # The only attribute that must have changed is the content.
            result[-1] = map_region(region.offset, region.size, region.mutable)

        else:
            raise ValueError('Invalid action: {0}'.format(action))

    return result, f.getvalue()


def format_region(i, filename, region):
    result = [
        '== {0} - {1}: region from {2.offset} to {2.last} ==',
        '\nSize: {2.size}',
    ]
    if region.size:
        result.append('\nMutable: {3}')
    result.append('\n{2.content}\n\n')

    return ''.join(result).format(
        i, filename, region,
        str(region.mutable).upper()
    )

def format_regions(filename, regions):
    return ''.join(
        format_region(i, filename, region)
        for i, region in enumerate(regions, 1)
    )


def actions_to_args(actions):
    """
    Convert a list of actions to arguments (a list of strings) to be provided
    to the test driver's command line.
    """

    def helper(action):
        return [type(action).__name__.lower()] + [
            str(field) for field in action
        ]

    return sum(map(helper, actions), [])



def returnString():
    return "A string"

def returnIntList():
    return [12, 34]

def returnStrList():
    return ["str1", "str2"]


class NewStyleClass(object):
    pass

class OldStyleClass:
    pass

def returnNewInstance():
    return NewStyleClass()

def returnOldInstance():
    return OldStyleClass()

def raiseError():
    raise Exception("Message")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("json1")
    def test_json1(self):
        self.gprbuild()
        self.runexec("decode", "test.out")

    @chdir("M605-001")
    def test_M605_001(self):
        self.gprbuild()
        self.runexec("read", "test.expected")

    @chdir("M626_041")
    def test_M626_041(self):
        self.gprbuild()
        self.runexec("obj/json_test", "test.out")

    @chdir("MB28-001")
    def test_MB28_001(self):
        self.runexec(["python", "make_json.py"])
        self.gprbuild()
        self.runexec("json_stack_test", "test.out")

    @chdir("utf8")
    def test_utf8(self):
        self.gprbuild()
        self.runexec("obj/json_utf8", "")



import unittest
from support import GNATCOLL_TestCase, chdir, pathsep


class Test(GNATCOLL_TestCase):
    @chdir("templates1")
    def test_templates1(self):
        self.gprbuild()
        self.runexec("obj/templates", "")
